import re

from .source import get_source
from .scheme import get_scheme
from .exceptions import RegistrationError


def register_file(config: dict, path: str):
    source = get_source(config, path)
    scheme = get_scheme(config, path)
    context = scheme.get_context(source, path)

    for pre_handler in get_pre_handlers(config):
        pre_handler(config, path, context)

    for backend in get_backends(config):
        if backend.exists(source, context):
            if config.replace:
                backend.register(source, context, replace=True)
            else:
                raise RegistrationError(f'Object {context} is already registered')
        else:
            backend.register(source, context, replace=False)

    for post_handler in get_post_handlers(config):
        post_handler(config, path, context)


def get_pre_handlers(config):
    pass


def get_post_handlers(config):
    pass


def get_backends(config):
    pass
