from tempfile import NamedTemporaryFile
from dataclasses import dataclass, field
from typing import Union, Type, Optional, List, Callable, Any

import lxml.etree

from .source import Source


def read_xml(source: Source, path: str) -> lxml.etree._ElementTree:
    with NamedTemporaryFile() as f:
        source.get_file(path, f.name)
        return lxml.etree.parse(f)

@dataclass
class Parameter:
    xpath: str
    multi: bool = False
    parser: Optional[Callable[[str], Any]] = None
    namespaces: dict = field(default_factory=dict)


def parse_metadata_schema(tree: lxml.etree._ElementTree, schema: dict, namespaces: dict=None) -> dict:
    out = {}
    for key, param in schema.items():
        values = tree.xpath(param.xpath, namespaces=param.namespaces or namespaces)
        if param.multi:
            value = [
                param.parser(v) if param.parser else v
                for v in values
            ]
        else:
            value = param.parser(values[0]) if param.parser else values[0]

        out[key] = value

    return out
